#include "xcanfd.h"

XCanFd_Config XCanFd_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"xlnx,canfd-2.0", /* compatible */
		0xf19e0000, /* reg */
		0x0, /* xlnx,rx-mode */
		0x30, /* xlnx,num-of-rx-mb-buf */
		0x20, /* xlnx,num-of-tx-buf */
		0x401b, /* interrupts */
		0xe2000000 /* interrupt-parent */
	},
	{
		"xlnx,canfd-2.0", /* compatible */
		0xf19f0000, /* reg */
		0x0, /* xlnx,rx-mode */
		0x30, /* xlnx,num-of-rx-mb-buf */
		0x20, /* xlnx,num-of-tx-buf */
		0x401c, /* interrupts */
		0xe2000000 /* interrupt-parent */
	},
	{
		"xlnx,canfd-2.0", /* compatible */
		0xf1a00000, /* reg */
		0x0, /* xlnx,rx-mode */
		0x30, /* xlnx,num-of-rx-mb-buf */
		0x20, /* xlnx,num-of-tx-buf */
		0x405f, /* interrupts */
		0xe2000000 /* interrupt-parent */
	},
	{
		"xlnx,canfd-2.0", /* compatible */
		0xf1a10000, /* reg */
		0x0, /* xlnx,rx-mode */
		0x30, /* xlnx,num-of-rx-mb-buf */
		0x20, /* xlnx,num-of-tx-buf */
		0x4060, /* interrupts */
		0xe2000000 /* interrupt-parent */
	},
	 {
		 NULL
	}
};